import 'dart:async';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:firebase_auth/firebase_auth.dart';

class ChatDBFireStore {


  static String getDocName() {
    String dbUser = "users";
    return dbUser;
  }

  setUserLastSeen(User? logInUser) async {
    if (logInUser != null)
      await FirebaseFirestore.instance
          .collection(getDocName())
          .doc(logInUser.uid)
          .set({
        'lastActive': DateTime.now().millisecondsSinceEpoch,
        'online': 'online'
      }, SetOptions(merge: true));
  }

  void setChatLastSeen(
      var currentUserId, var stCollection, var chatId, bool isChat) async {
    //if (logInUser != null) {
    await FirebaseFirestore.instance.collection(stCollection).doc(chatId).set(
        {currentUserId: isChat ? true : DateTime.now().millisecondsSinceEpoch},
        SetOptions(merge: true));
    //}
  }

  void setChatWith(String? id, String? peerId) {
    FirebaseFirestore.instance
        .collection(getDocName())
        .doc(id)
        .update({'chattingWith': peerId});
  }

  List<dynamic>? friendList;
  Future<List<dynamic>?> getAllFriendList(
      var friendId, var currentUserId) async {
    FirebaseFirestore.instance
        .collection(getDocName())
        .doc(friendId)
        .get()
        .then((DocumentSnapshot documentSnapshot) {
      if (documentSnapshot.exists) {
        friendList = documentSnapshot.get('friends');
        if (friendList!.contains(currentUserId)) {
        } else {
          friendList!.add(currentUserId);
          FirebaseFirestore.instance
              .collection(getDocName())
              .doc(friendId)
              .update({"friends": friendList}).whenComplete(() => {});
        }

        return friendList;
      } else {
        return friendList;
        //print('Document does not exist on the database');
      }
    });
  }

  static Future<void> checkUserExists(User logInUser) async {
    late QuerySnapshot result;
    try {
      result = await FirebaseFirestore.instance
          .collection(getDocName())
          .where('userId', isEqualTo: logInUser.uid)
          .get();
    } catch (e) {
      print('ex ' + e.toString());
    }

    final List<DocumentSnapshot> documents = result.docs;
    if (documents.length == 0) {
      // Update data to server if new user
      await saveNewUser(logInUser);
    }
  }

  static saveNewUser(User logInUser) {
    List<String> friendList = [];

    FirebaseFirestore.instance.collection(getDocName()).doc(logInUser.uid).set({
      'nickname': logInUser.displayName,
      'photoUrl': logInUser.photoURL,
      'userId': logInUser.uid,
      'email': logInUser.email,
      'friends': friendList,
      'createdAt': DateTime.now().millisecondsSinceEpoch.toString(),
      'chattingWith': null,
      'online': null
    });
  }

  static streamChatData() {
    FirebaseFirestore.instance
        .collection(ChatDBFireStore.getDocName())
        .snapshots();
  }

  static Future<void> makeUserOnline(User logInUser) async {
    await ChatDBFireStore().setUserLastSeen(logInUser);

    /**
     * Use online offline feature
     * uncomment below code
     * and firebase_database: in pubspec
     */
    // FirebaseDatabase.instance
    //     .reference()
    //     .child("/status/" + logInUser.uid)
    //     .onDisconnect()
    //     .set("offline");
    // FirebaseDatabase.instance
    //     .reference()
    //     .child("/status/" + logInUser.uid)
    //     .set("online");
  }

  Stream<QuerySnapshot> streamChatDataList(var stCollection, var groupChatId) {
    return FirebaseFirestore.instance
        .collection(stCollection)
        .doc(groupChatId)
        .collection(groupChatId)
        .orderBy('timestamp', descending: true)
        .limit(50)
        .snapshots();
  }

  Future<List<dynamic>?> getFriendList(var userId) async {
    List<dynamic>? friendList;

    FirebaseFirestore.instance
        .collection(getDocName())
        .doc(userId)
        .get()
        .then((DocumentSnapshot documentSnapshot) {
      if (documentSnapshot.exists) {
        friendList = documentSnapshot.get('friends');
        return friendList;
      } else {
        return null;
        print('Document does not exist on the database');
      }
    });

    return friendList;
  }
}
