import 'dart:async';
import 'dart:io';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:image_picker/image_picker.dart';
import '../chatDB.dart';
import '../chatData.dart';
import '../chatWidget.dart';
import '../constants.dart';

class Chat extends StatelessWidget {
  final String? peerId;
  final String? peerAvatar;
  final String? peerName;
  final String currentUserId;
  static const String id = "chat";

  Chat(
      {Key? key,
      required this.currentUserId,
      required this.peerId,
      required this.peerAvatar,
      required this.peerName})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    return new Scaffold(
      backgroundColor: Colors.white,
      appBar: ChatWidget.getAppBar(peerName!),
      body: new _ChatScreen(
        currentUserId: currentUserId,
        peerId: peerId,
        peerAvatar: peerAvatar,
      ),
    );
  }
}

class _ChatScreen extends StatefulWidget {
  final String? peerId;
  final String? peerAvatar;
  final String currentUserId;

  _ChatScreen(
      {Key? key,
      required this.peerId,
      required this.peerAvatar,
      required this.currentUserId})
      : super(key: key);

  @override
  State createState() =>
      new _ChatScreenState(peerId: peerId, peerAvatar: peerAvatar);
}

class _ChatScreenState extends State<_ChatScreen> {
  _ChatScreenState({Key? key, required this.peerId, required this.peerAvatar});

  String? peerId;
  String? peerAvatar;
  String? id;

  var listMessage;
  String? groupChatId;

  File? imageFile;
  late bool isLoading;
  bool? isShowSticker;
  late String imageUrl;

  final TextEditingController textEditingController =
      new TextEditingController();
  final ScrollController listScrollController = new ScrollController();
  final FocusNode focusNode = new FocusNode();
  var stCollection = 'messages';
  bool isOGroup = false;

  var adminId = 'YRTZ3lzw13MKr0mFMa613Uuxe6r1';
  var acId = '';
  List<dynamic>? friendList;

  @override
  void initState() {
    super.initState();

    groupChatId = '';

    isLoading = false;
    isShowSticker = false;
    imageUrl = '';

    readLocal();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    if (state == AppLifecycleState.resumed) {
      ChatDBFireStore().setChatLastSeen(
          widget.currentUserId, stCollection, groupChatId, true);
    } else {
      ChatDBFireStore().setChatLastSeen(
          widget.currentUserId, stCollection, groupChatId, false);
    }
  }

  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
    ChatDBFireStore().setChatLastSeen(
        widget.currentUserId, stCollection, groupChatId, false);
  }

  void onFocusChange() {
    if (focusNode.hasFocus) {
      // Hide sticker when keyboard appear
      setState(() {
        isShowSticker = false;
      });
    }
  }

  readLocal() async {
    id = widget.currentUserId ?? '';
    groupChatId = ChatData.getGroupChatID(id, peerId);

    setState(() async {
      friendList = await ChatDBFireStore()
          .getAllFriendList(peerId, widget.currentUserId);
    });
    ChatDBFireStore().setChatWith(id, peerId);
    setState(() {});
  }

  getImage(int index) async {
    PickedFile? selectedFile;
    selectedFile = await ChatData().getImage(index);
    if (selectedFile != null) {
      setState(() {
        //orFile=selectedFile;
        isLoading = true;
      });
      uploadFile(selectedFile);
    }
  }

  uploadFile(PickedFile orFile) async {
    imageUrl = await ChatData().uploadFile(orFile);
    if (imageUrl != null && imageUrl != '') {
      setState(() {
        isLoading = false;
        onSendMessage(imageUrl, 1);
      });
    } else {
      setState(() {
        isLoading = false;
      });
    }
  }

  void onSendMessage(String content, int type) async {
    // type: 0 = text, 1 = image, 2 = sticker
    if (content.trim() != '') {
      textEditingController.clear();

      var documentReference = FirebaseFirestore.instance
          .collection(stCollection)
          .doc(groupChatId)
          .collection(groupChatId!)
          .doc(DateTime.now().millisecondsSinceEpoch.toString());

      FirebaseFirestore.instance.runTransaction((transaction) async {
        await transaction.set(
          documentReference,
          {
            'idFrom': acId == '5' ? adminId : widget.currentUserId,
            'idTo': peerId,
            'timestamp': DateTime.now().millisecondsSinceEpoch.toString(),
            'content': content,
            'type': type
          },
        );
      });
      listScrollController.animateTo(0.0,
          duration: Duration(milliseconds: 300), curve: Curves.easeOut);
    } else {
      //Fluttertoast.showToast(msg: 'Nothing to send');
    }
  }

  Future<bool> onBackPress() {
    Navigator.pop(context);
    return Future.value(false);
  }

  @override
  Widget build(BuildContext context) {
    return MaterialApp(home: Builder(builder: (BuildContext context) {
      return Scaffold(
          body: WillPopScope(
        child: Stack(
          children: <Widget>[
            Column(
              children: <Widget>[
                // List of messages
                ChatWidget.widgetChatBuildListMessage(
                    groupChatId,
                    listMessage,
                    widget.currentUserId,
                    peerAvatar,
                    listScrollController,
                    stCollection),

                buildInput(),
              ],
            ),
            // Loading
            buildLoading()
          ],
        ),
        onWillPop: onBackPress,
      ));
    }));
  }

  Widget buildLoading() {
    return Positioned(
      child: isLoading
          ? Container(
              child: Center(
                child: CircularProgressIndicator(
                    valueColor: AlwaysStoppedAnimation<Color>(themeColor)),
              ),
              color: Colors.white.withOpacity(0.8),
            )
          : Container(),
    );
  }

  Widget buildInput() {
    return Container(
      child: Row(
        children: <Widget>[
          // Button send image
          Material(
            child: new Container(
              margin: new EdgeInsets.symmetric(horizontal: 1.0),
              child: new IconButton(
                icon: new Icon(Icons.image),
                onPressed: () => getImage(0),
                color: primaryColor,
              ),
            ),
            color: Colors.white,
          ),
          Visibility(
            visible: !kIsWeb,
            child: Material(
              child: new Container(
                margin: new EdgeInsets.symmetric(horizontal: 1.0),
                child: new IconButton(
                  icon: new Icon(Icons.camera_alt),
                  onPressed: () => getImage(1),
                  color: primaryColor,
                ),
              ),
              color: Colors.white,
            ),
          ),
          // Edit text
          Flexible(
            child: Container(
              child: TextField(
                style: TextStyle(color: primaryColor, fontSize: 15.0),
                controller: textEditingController,
                decoration: InputDecoration.collapsed(
                  hintText: 'Type your message...',
                  hintStyle: TextStyle(color: greyColor),
                ),
                focusNode: focusNode,
              ),
            ),
          ),

          // Button send message
          Material(
            child: new Container(
              margin: new EdgeInsets.symmetric(horizontal: 8.0),
              child: new IconButton(
                icon: new Icon(Icons.send),
                onPressed: () => onSendMessage(textEditingController.text, 0),
                color: primaryColor,
              ),
            ),
            color: Colors.white,
          ),
        ],
      ),
      width: double.infinity,
      height: 50.0,
      decoration: new BoxDecoration(
          border:
              new Border(top: new BorderSide(color: greyColor2, width: 0.5)),
          color: Colors.white),
    );
  }
}
